'use strict';
/**
 * Start the requirejs config.
 */
require.config({
  paths: {
    'bootstrap-switch': '../../webjars/bootstrap-switch/3.3.2/js/bootstrap-switch.min'
  },
  shim: {
    'bootstrap-switch' : ['jquery']
  }
});
/**
 * Start the real script.
 */
require(['jquery', 'AddCustomNotificationFilterPreferenceLivetable', 'bootstrap', 'bootstrap-switch', 'xwiki-events-bridge'], function ($, AddCustomNotificationFilterPreferenceLivetable) {

  /**
   * Page initialization
   */
  $(document).ready(function() {

    new AddCustomNotificationFilterPreferenceLivetable($('#modal-add-custom-filter-preference'), window['livetable_notificationCustomFilterPreferencesLiveTable'], $('.customFilterPreferences button.btn-addfilter'),
      $('.customFilterPreferences').attr('data-doc-url'));
    var initBootstrapSwitches = function () {
      $('#notificationCustomFilterPreferencesLiveTable .notificationFilterPreferenceCheckbox, #notificationCustomFilterPreferencesLiveTable .toggleableFilterPreferenceCheckbox').bootstrapSwitch({
        size: 'mini',
        labelText: '<img src=\"..\/..\/resources\/icons\/silk\/bell.png\" alt=\"Icon\" \/>'
      });
    };
    $(document).on('xwiki:livetable:displayComplete', function (event, data) {
      // Enable bootstrap switches
      initBootstrapSwitches();
    });
    initBootstrapSwitches();

  });
});

define('AddCustomNotificationFilterPreferenceLivetable', ['jquery', 'xwiki-meta', 'bootstrap'], function($, xm) {

  return function(modal, filterPreferencesLivetable, buttonAddFilter, docURL) {
    var self = this;
    self.modal = modal;
    self.filterPreferencesLivetable = filterPreferencesLivetable;
    self.buttonAddFilter = buttonAddFilter;
    self.docURL = docURL;
    self.filterTypeSelector = modal.find('select#notificationFilterTypeSelector');
    self.eventTypeSelector = modal.find('select#notificationFilterEventTypeSelector');
    self.filterScopeTree = modal.find('.location-tree');
    self.cancelButton = modal.find('button[data-action="cancel"]');
    self.submitButton = modal.find('button[data-action="submit"]');
    
    self.getFormats = function () {
      return self.modal.find('input[name=notificationFilterNotificationFormatSelector]:checked').map(function () { return $(this).val();}).get().join(',');
    };
    
    /**
     * Initialization
     */
    self.init = function () {
      self.buttonAddFilter.click(function() {
        self.filterScopeTree.xtree();
        self.submitButton.prop('disabled', 'disabled');

        self.filterScopeTree.on('changed.jstree', function(e, data) {
          if (data.selected.length > 0 && self.getFormats() !== "") {
            self.submitButton.prop('disabled', '');
          }
        });
      });
      

      self.submitButton.click(self.onSelectButtonClicked);
    };

    /**
     * On "submit" button clicked
     */
    self.onSelectButtonClicked = function() {
      self.modal.modal('toggle');

      var tree = $.jstree.reference(self.filterScopeTree);
      var nodes = tree.get_selected(true);

      var serviceReference = XWiki.Model.resolve('XWiki.Notifications.Code.NotificationPreferenceService', XWiki.EntityType.DOCUMENT);
      var serviceURL = new XWiki.Document(serviceReference).getURL('get', 'outputSyntax=plain');
      var eventTypesSelected = self.eventTypeSelector.val();
      var eventTypes;
      if (eventTypesSelected != null && eventTypesSelected.length > 0) {
        eventTypes = eventTypesSelected.join(',');
      }
      var target = self.modal.data('target');
      
      var params = {
        action: 'createScopeFilterPreference',
        target: target,
        filterType: self.filterTypeSelector.val(),
        filterFormats: self.getFormats(),
        eventTypes: eventTypes,
        csrf: xm.form_token,
        wiki: [],
        space: [],
        page: []
      }
      if (target == 'user') {
        params['user'] = self.modal.data('user');
      }

      for (var i = 0; i < nodes.length; ++i) {
        var node = nodes[i];
        var reference = node.data.id;
        if (node.data.type == 'wiki') {
          params['wiki'].push(reference);
        } else if (node.data.type == 'document' && node.data.id.match('WebHome$')) {
          params['space'].push(reference.substring(0, reference.length - '.WebHome'.length));
        } else {
          params['page'].push(reference);
        }
      }

      // Saving
      var notification = new XWiki.widgets.Notification("\u0412\u0430\u0448\u0438 \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0441\u043E\u0445\u0440\u0430\u043D\u044F\u044E\u0442\u0441\u044F ...", 'inprogress');
      // We use a "traditional: true" argument here to be sure that the argument names are not transformed to wiki[] etc by jquery. For more info see doc about $.param.
      $.post({
        url: serviceURL,
        data: params,
        traditional: true
      }).done(function (data) {
        notification.hide();
        new XWiki.widgets.Notification("\u0421\u043E\u0445\u0440\u0430\u043D\u0435\u043D\u043E!", 'done');

        // Reload the livetable
        self.filterPreferencesLivetable.refresh();
      }).fail(function () {
        notification.hide();
        new XWiki.widgets.Notification("\u041D\u0435 \u0443\u0434\u0430\u043B\u043E\u0441\u044C \u0441\u043E\u0445\u0440\u0430\u043D\u0438\u0442\u044C \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438", 'error');
      });
    };

    // Call init()
    self.init();
  };

});

define('CustomToggleableFilterPreference', ['jquery', 'bootstrap-switch'], function($) {
  /**
  * Construct the Toggleable Filter Preference object.
  */
  return function(domElement, docURL, row) {
    var self               = this;
    self.domElement        = $(domElement);
    self.docURL           = docURL;
    self.row               = row;
    /**
     * Initialization
     */
    self.init = function () {
      // On change
      self.domElement.off('switchChange.bootstrapSwitch.customFilter')
        .on('switchChange.bootstrapSwitch.customFilter', function(event, state) {
          setTimeout(function() { self.save(state); }, 1);
        });
    };

    /**
     * Save the filter preference
     */
    self.save = function(state) {
      var notification = new XWiki.widgets.Notification("\u0412\u0430\u0448\u0438 \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0441\u043E\u0445\u0440\u0430\u043D\u044F\u044E\u0442\u0441\u044F ...", 'inprogress');
      var filterName = self.domElement.attr('data-filterName');
      var objectNumber = self.domElement.attr('data-objectNumber');
      if (objectNumber == '') {
        // If the object does not exist yet, create it (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.docURL + '/objects?media=json&minorRevision=true';
        var params = {
          'className': 'XWiki.Notifications.Code.ToggleableFilterPreferenceClass',
          'property#filterName': filterName,
          'property#isEnabled': state ? '1' : '0'
        };
        $.post(restURL, params).done(function (data) {
          self.domElement.attr('data-objectNumber', data.number);
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430 \u0441\u043E\u0445\u0440\u0430\u043D\u0435\u043D\u044B!", 'done');
        }).fail(function () {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0435 \u0443\u0434\u0430\u043B\u043E\u0441\u044C \u0441\u043E\u0445\u0440\u0430\u043D\u0438\u0442\u044C \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430", 'error');
        });
      } else if (objectNumber != '') {
        // If the object already exist, just update its value (make this change a minor edit so it's not displayed, by default, in notifications)
        var restURL = self.docURL + '/objects/XWiki.Notifications.Code.ToggleableFilterPreferenceClass/' + objectNumber
          + '/properties/isEnabled?media=json&minorRevision=true';
        $.ajax(restURL, {
          method: 'PUT',
          contentType: 'text/plain',
          data: state ? '1' : '0'
        }).done(function (data) {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430 \u0441\u043E\u0445\u0440\u0430\u043D\u0435\u043D\u044B!", 'done');
          $(document).trigger('xwiki:livetable:notifications-filters-custom:toggle', {'row': self.row, 'state': state});
        }).fail(function () {
          notification.hide();
          new XWiki.widgets.Notification("\u041D\u0435 \u0443\u0434\u0430\u043B\u043E\u0441\u044C \u0441\u043E\u0445\u0440\u0430\u043D\u0438\u0442\u044C \u043D\u0430\u0441\u0442\u0440\u043E\u0439\u043A\u0438 \u0444\u0438\u043B\u044C\u0442\u0440\u0430", 'error');
        });
      }
    };

    // Call init
    self.init();
  };
});

require([
  'jquery',
  'xwiki-meta',
  'CustomToggleableFilterPreference',
  'bootstrap-switch',
  'xwiki-events-bridge'
], function ($, xm, CustomToggleableFilterPreference) {

  $(document).on('xwiki:livetable:notifications-filters-custom:toggle', function(event, data) {
    var changedRow = $(data.row.isEnabled);
    if (changedRow.length > 0) {
      if (data.state) {
        changedRow.attr("checked", "checked");
      } else {
        changedRow.removeAttr("checked");
      }
      data.row.isEnabled = changedRow.get(0).outerHTML;
    }
  });

  // Globals
  var serviceReference = XWiki.Model.resolve('XWiki.Notifications.Code.NotificationPreferenceService',
    XWiki.EntityType.DOCUMENT);
  var serviceURL = new XWiki.Document(serviceReference).getURL('get', 'outputSyntax=plain');
  var target = $('.customFilterPreferences').attr('data-target');

  var docURL = $('.customFilterPreferences').attr('data-doc-url');
  var postParams = {
    csrf: xm.form_token,
    target: target
  };
  if (target == 'user') {
    var targetUser = $('.customFilterPreferences').attr('data-user');
    postParams['user'] = targetUser;
  };

  var enhanceRow = function(row, data) {
    // Delete action
    $(row).find('a.actiondelete').off('click.deleteCustomFilter').on('click.deleteCustomFilter', function(event) {
      event.preventDefault();
      new XWiki.widgets.ConfirmationBox({
        onYes: function() {
          var notif = new XWiki.widgets.Notification("Удаление...", 'inprogress');
          var params = {
            action: 'deleteFilterPreference',
            filterPreferenceId: data.filterPreferenceId
          };
          $.post(serviceURL, $.extend({}, params, postParams)).done(function() {
            notif.replace(new XWiki.widgets.Notification("Настройки фильтра удалены!", 'done'));
            window.livetable_notificationCustomFilterPreferencesLiveTable.refresh();
          }).fail(function() {
            notif.replace(new XWiki.widgets.Notification("Не удалось удалить настройки фильтра", 'error'));
          });
        }
      });
    });
    // Enable / Disable action for classic filter preferences
    $(row).find('.notificationFilterPreferenceCheckbox').off('switchChange.bootstrapSwitch.customFilter')
    .on('switchChange.bootstrapSwitch.customFilter', function(event, state) {
      var notif = new XWiki.widgets.Notification("Сохранение...", 'inprogress');
      var params = {
        action: 'setFilterPreferenceEnabled',
        filterPreferenceId: data.filterPreferenceId,
        enabled: state
      };
      $.post(serviceURL, $.extend({}, params, postParams)).done(function() {
        notif.replace(new XWiki.widgets.Notification("Настройки фильтра сохранены!", 'done'));
        $(document).trigger('xwiki:livetable:notifications-filters-custom:toggle', {'row': data, 'state': state});
      }).fail(function() {
        notif.replace(new XWiki.widgets.Notification("Не удалось сохранить настройки фильтра", 'error'));
      });
    });
    // Enable / Disable action for toggleable filter preferences
    $(row).find('.toggleableFilterPreferenceCheckbox').each(function() {
      new CustomToggleableFilterPreference(this, docURL, data);
    });
  };

  // Callback on livetable row printing
  $(document).on('xwiki:livetable:notificationCustomFilterPreferencesLiveTable:newrow', function(event, data) {
    enhanceRow(data.row, data.data);
  });

  // Initializer
  $(function() {
    // This script is usually loaded after the live table is displayed and so the previous callback might not be called.
    $('#notificationCustomFilterPreferencesLiveTable-display > tr[data-index]').each(function() {
      var data = window.livetable_notificationCustomFilterPreferencesLiveTable.fetchedRows[$(this).data('index')];
      enhanceRow(this, data);
    });
  });
});
